<?php
/**
 * ownCloud - user_cas
 *
 * @author Sixto Martin <sixto.martin.garcia@gmail.com>
 * @copyright Sixto Martin Garcia. 2012
 * @copyright Leonis. 2014 <devteam@leonis.at>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU AFFERO GENERAL PUBLIC LICENSE for more details.
 *
 * You should have received a copy of the GNU Affero General Public
 * License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

/**
 * This class contains all hooks.
 */
 
use \OCA\user_ldap\lib\ldap;
use \OCP\IDBConnection;
use \OCA\User_LDAP\Mapping\UserMapping;
use OCA\DAV\CalDAV\CalDavBackend;
use OCA\DAV\Connector\Sabre\Principal;
use OCA\DAV\Files\CustomPropertiesBackend;
use OCP\IUserManager;

class OC_USER_CAS_Hooks {

	static public function post_login($parameters) {

		$uid = $parameters['uid'];
		$casBackend = OC_USER_CAS::getInstance();
		$userDatabase = new OC_User_Database;

		if (eolephpCAS::isAuthenticated()) {
			// Récupération des attribut phpCAS
			$attributes = array();
			$cas_attributes = eolephpCAS::getAttributes();
			$cas_attributes = $cas_attributes["utilisateur"]; 
			$cas_uid = eolephpCAS::getUser();
            
            // Initialisation d'une connection LDAP
			$ldapWrapper = new OCA\user_ldap\lib\LDAP();
            $ldapHelper =  new OCA\user_ldap\lib\Helper();
			$configPrefixes = $ldapHelper->getServerConfigurationPrefixes(true);
			$ocConfig = \OC::$server->getConfig();
            $dbc = \OC::$server->getDatabaseConnection();


            $userMapper  = new UserMapping($dbc);

			$userManager = new OCA\user_ldap\lib\user\Manager($ocConfig,new OCA\user_ldap\lib\FilesystemHelper(),new OCA\user_ldap\lib\LogWrapper(),\OC::$server->getAvatarManager(),new \OCP\Image(),$dbc,\OC::$server->getUserManager());
			$connector = new OCA\user_ldap\lib\Connection($ldapWrapper, $configPrefixes[0]);
			$ldapAccess = new OCA\user_ldap\lib\Access($connector, $ldapWrapper, $userManager);
			$userBackend  = new OCA\user_ldap\USER_LDAP($ldapAccess,$ocConfig);
            $ldapAccess->setUserMapper($userMapper);

			$dn=$ldapAccess->searchUsers("uid=".$cas_uid ,"dn",10,0);
			if(array($dn)) {
				$ldapAccess->dn2ocname($dn[0]["dn"][0],$cas_uid,true);
			}			
			
			if (array_key_exists($casBackend->displayNameMapping, $cas_attributes)) {
				$attributes['cas_name'] = $cas_attributes[$casBackend->displayNameMapping][0];	
			}

			if (array_key_exists($casBackend->mailMapping, $cas_attributes)) {
				$attributes['cas_email'] = $cas_attributes[$casBackend->mailMapping][0];
			}


			if (array_key_exists($casBackend->groupMapping, $cas_attributes)) {
				$attributes['cas_groups'] = $cas_attributes[$casBackend->groupMapping];
			}
			else if (!empty($casBackend->defaultGroup)) {
				$attributes['cas_groups'] = array($casBackend->defaultGroup);
			}

			if (!$userDatabase->userExists($cas_uid)) {
				// create users if they do not exist
				if (preg_match( '/[^a-zA-Z0-9 _\.@\-]/', $cas_uid)) {
					return false;
				}
				else {
					$random_password = \OCP\Util::generateRandomBytes(20);
					$userDatabase->createUser($cas_uid, $random_password);

					// after creating the user, fill the attributes
					if($userDatabase->userExists($cas_uid)) 
						OC_USER_CAS_Hooks::update_user($cas_uid,$attributes);
					
				}
			}
			
			
			// try to update user attributes
			if ($casBackend->updateUserData) 
				OC_USER_CAS_Hooks::update_user($cas_uid,$attributes);

			// Génération calendrier par défaut
			$userManager = \OC::$server->getUserManager();
			$groupManager = \OC::$server->getGroupManager();
			$dbConnection = \OC::$server->getDatabaseConnection();
			$principalBackend = new Principal($userManager,$groupManager);
			$caldav = new CalDavBackend($dbConnection, $principalBackend);

			$name = "Personnel";
			if(!$david=$caldav->getCalendarByUri("principals/users/$cas_uid",$name)) {
				$caldav->createCalendar("principals/users/$cas_uid", $name, []);
				$david=$caldav->getCalendarByUri("principals/users/$cas_uid",$name);
			}

			// Rendre visible quoi qu'il arrive le calendrier Personnel à la connexion
			if($david!="") {
				$sql = "SELECT id FROM `*PREFIX*properties` WHERE userid=? AND propertypath=? AND propertyname=?";
				$result = $dbConnection->executeQuery($sql,array($cas_uid,"calendars/$cas_uid/Personnel","{http://owncloud.org/ns}calendar-enabled"));
				if($row=$result->fetch()) {
					$sql = "UPDATE `*PREFIX*properties` SET propertyvalue=1 WHERE userid=? AND propertypath=? AND propertyname=?";
					$result = $dbConnection->executeQuery($sql,array($cas_uid,"calendars/$cas_uid/Personnel","{http://owncloud.org/ns}calendar-enabled"));
				}
				else {
					$sql = "INSERT INTO `*PREFIX*properties` (userid,propertypath,propertyname,propertyvalue) VALUES (?,?,?,1)";
					$result = $dbConnection->executeQuery($sql,array($cas_uid,"calendars/$cas_uid/Personnel","{http://owncloud.org/ns}calendar-enabled"));
				}
			}

			return true;
		}
		return false;
	}

	static public function update_user($uid, $attributes) {

		$casBackend = OC_USER_CAS::getInstance();

		if(isset($attributes['cas_email'])) {
			update_mail($uid, $attributes['cas_email']);
		}
		if (isset($attributes['cas_name'])) {
			update_name($uid, $attributes['cas_name']);
		}
		if (isset($attributes['cas_groups'])) {
			update_groups($uid, $attributes['cas_groups'], $casBackend->protectedGroups, false);
		}
	}


	static public function logout($parameters) {
		$casBackend = OC_USER_CAS::getInstance();

		if (eolephpCAS::isAuthenticated()) 
			$urllogout="https://".$_SERVER["SERVER_NAME"]."/owncloud";
			eolephpCAS::logoutWithUrl($urllogout);
		
		return true;
	}

}

function update_mail($uid, $email) {
    $config = \OC::$server->getConfig();
    if ($email != $config->getUserValue($uid, 'settings', 'email', '')) {
        $config->setUserValue($uid, 'settings', 'email', $email);
    }
}

function update_name($uid, $name) {
		$casBackend = OC_USER_CAS::getInstance();
		$casBackend->setDisplayName($uid, $name);
}

/**
* Gets an array of groups and will try to add the group to OC and then add the user to the groups.
* 
*/
function update_groups($uid, $groups, $protected_groups=array(), $just_created=false) {

	if(!$just_created) {
		$old_groups = OC_Group::getUserGroups($uid);
		foreach($old_groups as $group) {
			if(!in_array($group, $protected_groups) && !in_array($group, $groups)) {
				OC_Group::removeFromGroup($uid,$group);
			}
		}
	}

	foreach($groups as $group) {
		if (!preg_match( '/[^a-zA-Z0-9 _\.@\-]/', $group)) {
			if (!OC_Group::inGroup($uid, $group)) {
				if (!OC_Group::groupExists($group)) {
					OC_Group::createGroup($group);
				}
				OC_Group::addToGroup($uid, $group);
			}
		}
	}
}
